#!/usr/bin/env python

# create pipeline
#
# create sphere to color
sphere = vtk.vtkSphereSource()
sphere.SetThetaResolution(20)
sphere.SetPhiResolution(40)
def colorCells ():
    randomColorGenerator = vtk.vtkMath()
    input = randomColors.GetInput()
    output = randomColors.GetOutput()
    numCells = input.GetNumberOfCells()
    colors = vtk.vtkFloatArray()
    colors.SetNumberOfTuples(numCells)
    i = 0
    while i < numCells:
        colors.SetValue(i,randomColorGenerator.Random(0,1))
        i = i + 1

    output.GetCellData().CopyScalarsOff()
    output.GetCellData().PassData(input.GetCellData())
    output.GetCellData().SetScalars(colors)
    del colors
    #reference counting - it's ok
    del randomColorGenerator

# Compute random scalars (colors) for each cell
randomColors = vtk.vtkProgrammableAttributeDataFilter()
randomColors.SetInputConnection(sphere.GetOutputPort())
randomColors.SetExecuteMethod(colorCells)
# mapper and actor
mapper = vtk.vtkPolyDataMapper()
mapper.SetInput(randomColors.GetPolyDataOutput())
mapper.SetScalarRange(randomColors.GetPolyDataOutput().GetScalarRange())
sphereActor = vtk.vtkActor()
sphereActor.SetMapper(mapper)
# Create a scalar bar
scalarBar = vtk.vtkScalarBarActor()
scalarBar.SetLookupTable(mapper.GetLookupTable())
scalarBar.SetTitle("Temperature")
scalarBar.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
scalarBar.GetPositionCoordinate().SetValue(0.1,0.05)
scalarBar.SetOrientationToVertical()
scalarBar.SetWidth(0.8)
scalarBar.SetHeight(0.9)
scalarBar.SetLabelFormat("%-#6.3f")
# Test the Get/Set Position 
scalarBar.SetPosition(scalarBar.GetPosition())
# Create graphics stuff
# Create the RenderWindow, Renderer and both Actors
#
ren1 = vtk.vtkRenderer()
ren2 = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren1)
renWin.AddRenderer(ren2)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)
ren1.AddActor(sphereActor)
ren2.AddActor2D(scalarBar)
renWin.SetSize(160,160)
ren1.SetViewport(0,0,0.75,1.0)
ren2.SetViewport(0.75,0,1.0,1.0)
ren2.SetBackground(0.3,0.3,0.3)
# render the image
#
renWin.Render()
scalarBar.SetNumberOfLabels(8)
renWin.Render()
w2i = vtk.vtkWindowToImageFilter()
w2i.SetInput(renWin)
w2i.SetMagnification(2)
w2i.Update()
# copy the output
outputData = w2i.GetOutput().NewInstance()
outputData.DeepCopy(w2i.GetOutput())
ia = vtk.vtkImageMapper()
ia.SetInput(outputData)
scalarBar.ReleaseGraphicsResources(renWin)
sphereActor.ReleaseGraphicsResources(renWin)
ia.SetColorWindow(255)
ia.SetColorLevel(127.5)
ia2 = vtk.vtkActor2D()
ia2.SetMapper(ia)
renWin.SetSize(320,320)
ren2.RemoveViewProp(scalarBar)
ren1.RemoveViewProp(sphereActor)
ren1.AddActor(ia2)
renWin.RemoveRenderer(ren2)
ren1.SetViewport(0,0,1,1)
renWin.Render()
# prevent the tk window from showing up then start the event loop
outputData.UnRegister(None)
# --- end of script --
